@@ -1,14 +1,18 @@
-#  Copyright 2021 DAI Foundation
+# Copyright 2021 DAI FOUNDATION (the original version https://github.com/daifoundation/ethtx_ce)
+# Copyright 2021-2022 Token Flow Insights SA (modifications to the original software as recorded
+# in the changelog https://github.com/EthTx/ethtx/blob/master/CHANGELOG.md)
 #
-#  Licensed under the Apache License, Version 2.0 (the "License");
-#  you may not use this file except in compliance with the License.
-#  You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
 #
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#  See the License for the specific language governing permissions and
-#  limitations under the License.
+# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
+# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and limitations under the License.
+#
+# The product contains trademarks and other branding elements of Token Flow Insights SA which are
+# not licensed under the Apache 2.0 license. When using or reproducing the code, please remove
+# the trademark and/or other branding elements.
 
 import logging
 from typing import Optional, Dict, List
@@ -38,6 +42,18 @@ from .transfers import ABITransfersDecoder
 log = logging.getLogger(__name__)
 
 
+def prune_delegates(call: DecodedCall) -> DecodedCall:
+    while len(call.subcalls) == 1 and call.subcalls[0].call_type == "delegatecall":
+        _value = call.value
+        call = call.subcalls[0]
+        call.value = _value
+
+    for i, sub_call in enumerate(call.subcalls):
+        call.subcalls[i] = prune_delegates(sub_call)
+
+    return call
+
+
 class ABIDecoder(IABIDecoder):
     def decode_transaction(
         self,
@@ -206,6 +222,9 @@ class ABIDecoder(IABIDecoder):
             )
             raise e
 
+        # remove chained delegatecalls from the tree
+        prune_delegates(full_decoded_transaction.calls)
+
         full_decoded_transaction.status = True
 
         return full_decoded_transaction
