@@ -1,14 +1,19 @@
-#  Copyright 2021 DAI Foundation
+# Copyright 2021 DAI FOUNDATION (the original version https://github.com/daifoundation/ethtx_ce)
+# Copyright 2021-2022 Token Flow Insights SA (modifications to the original software as recorded
+# in the changelog https://github.com/EthTx/ethtx/blob/master/CHANGELOG.md)
 #
-#  Licensed under the Apache License, Version 2.0 (the "License");
-#  you may not use this file except in compliance with the License.
-#  You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
 #
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#  See the License for the specific language governing permissions and
-#  limitations under the License.
+# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
+# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and limitations under the License.
+#
+# The product contains trademarks and other branding elements of Token Flow Insights SA which are
+# not licensed under the Apache 2.0 license. When using or reproducing the code, please remove
+# the trademark and/or other branding elements.
+
 import logging
 from typing import Dict, List, Union
 
@@ -35,7 +40,9 @@ class DecoderService:
         self.web3provider: NodeDataProvider = web3provider
         self.default_chain: str = default_chain
 
-    def decode_transaction(self, chain_id: str, tx_hash: str) -> DecodedTransaction:
+    def decode_transaction(
+        self, chain_id: str, tx_hash: str, recreate_semantics: bool = False
+    ) -> DecodedTransaction:
 
         # verify the transaction hash
         tx_hash = tx_hash if tx_hash.startswith("0x") else "0x" + tx_hash
@@ -76,9 +83,13 @@ class DecoderService:
         log.info(
             "Semantics used in decoding %s: %s",
             tx_hash,
-            ", ".join(used_semantics) if used_semantics else "",
+            ", ".join(set(used_semantics)) if used_semantics else "",
         )
 
+        if recreate_semantics:
+            self.semantic_decoder.repository.delete_semantics(chain_id, used_semantics)
+            return self.decode_transaction(chain_id, tx_hash, False)
+
         return semantically_decoded_tx
 
     def get_proxies(
