@@ -1,20 +1,24 @@
-#  Copyright 2021 DAI Foundation
+# Copyright 2021 DAI FOUNDATION (the original version https://github.com/daifoundation/ethtx_ce)
+# Copyright 2021-2022 Token Flow Insights SA (modifications to the original software as recorded
+# in the changelog https://github.com/EthTx/ethtx/blob/master/CHANGELOG.md)
 #
-#  Licensed under the Apache License, Version 2.0 (the "License");
-#  you may not use this file except in compliance with the License.
-#  You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
 #
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#  See the License for the specific language governing permissions and
-#  limitations under the License.
+# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
+# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and limitations under the License.
+#
+# The product contains trademarks and other branding elements of Token Flow Insights SA which are
+# not licensed under the Apache 2.0 license. When using or reproducing the code, please remove
+# the trademark and/or other branding elements.
 
 import logging
 from datetime import datetime
 
+from ethtx.decoders.decoders.errors import ERRORS
 from ethtx.models.decoded_model import Argument
-from ethtx.models.semantics_model import ParameterSemantics
 
 log = logging.getLogger(__name__)
 
@@ -76,7 +80,7 @@ def decode_event_parameters(data, topics, abi, anonymous):
         for i, parameter in enumerate(amended_topics[1:]):
             if not amended_topics[i + 1]:
                 break
-            parameter_name = ""
+            parameter_name = f"topic_parameter_{i}"
             parameter_type = "unknown"
             parameter_value = (
                 amended_topics[i + 1]
@@ -90,9 +94,9 @@ def decode_event_parameters(data, topics, abi, anonymous):
 
         no_parameters = len(data) // 64
         for i in range(no_parameters):
-            parameter_name = ""
+            parameter_name = f"data_parameter_{i}"
             parameter_type = "unknown"
-            parameter_value = data[64 * i : 64 * (i + 1)]
+            parameter_value = data[64 * i: 64 * (i + 1)]
 
             data_parameters[i] = Argument(
                 name=parameter_name, type=parameter_type, value=parameter_value
@@ -121,7 +125,7 @@ def decode_event_parameters(data, topics, abi, anonymous):
 
 
 def decode_function_parameters(
-    input_data, output, abi, status=True, strip_signature=True
+        input_data, output, abi, status=True, strip_signature=True
 ):
     if strip_signature and len(input_data) >= 10:
         stripped_input_data = input_data[10:]
@@ -146,16 +150,15 @@ def decode_function_parameters(
     else:
         input_parameters = []
 
-    if not status and output[:10] == "0x08c379a0":
-        error_abi = ParameterSemantics(
-            parameter_name="Error",
-            parameter_type="string",
-            components=[],
-            indexed=False,
-            dynamic=True,
+    if not status and (error_sig := output[:10]) in ERRORS:
+        error_name = Argument(
+            name="__error",
+            type="string",
+            value=ERRORS[error_sig]["name"]
         )
-        error_parameters, _ = decode_struct(output[10:], [error_abi])
-        output_parameters = [Argument(**error_parameters[0])]
+        error_abi = ERRORS[error_sig]["abi"]
+        error_parameters, _ = decode_struct(output[10:], error_abi)
+        output_parameters = [error_name, *(Argument(**param) for param in error_parameters)]
     else:
         if abi:
             if abi.outputs and status and output == "0x":
@@ -188,67 +191,70 @@ def decode_function_parameters(
 
 # helper function to decode an argument value based on expected type
 def decode_static_argument(raw_value, argument_type):
-    decoded_value = raw_value
 
-    if decoded_value:
+    if not raw_value:
+        return raw_value
 
-        if argument_type == "address":
-            if len(raw_value) >= 40:
-                decoded_value = "0x" + raw_value[-40:]
-            else:
-                decoded_value = raw_value
+    if argument_type == "address":
+        if len(raw_value) >= 40:
+            decoded_value = "0x" + raw_value[-40:]
+        else:
+            decoded_value = raw_value
 
-        elif argument_type[:4] == "uint":
-            if isinstance(raw_value, str):
-                decoded_value = int(raw_value, 16)
-            else:
-                decoded_value = raw_value
+    elif argument_type[:4] == "uint":
+        if isinstance(raw_value, str):
+            decoded_value = int(raw_value, 16)
+        else:
+            decoded_value = raw_value
 
-        elif argument_type[:3] == "int":
-            if isinstance(raw_value, str):
-                decoded_value = int(raw_value, 16)
-                if decoded_value & (1 << (256 - 1)):
-                    decoded_value -= 1 << 256
-            else:
-                decoded_value = raw_value
+    elif argument_type[:3] == "int":
+        if isinstance(raw_value, str):
+            decoded_value = int(raw_value, 16)
+            if decoded_value & (1 << (256 - 1)):
+                decoded_value -= 1 << 256
+        else:
+            decoded_value = raw_value
 
-        elif argument_type == "bool":
-            if int(raw_value, 16) == 0:
-                decoded_value = "False"
-            else:
-                decoded_value = "True"
+    elif argument_type == "bool":
+        if int(raw_value, 16) == 0:
+            decoded_value = "False"
+        else:
+            decoded_value = "True"
 
-        elif argument_type == "bytes":
-            decoded_value = "0x" + bytes.fromhex(raw_value[2:]).hex()
+    elif argument_type == "bytes":
+        decoded_value = "0x" + bytes.fromhex(raw_value[2:]).hex()
 
-        elif argument_type[:5] == "bytes":
+    elif argument_type[:5] == "bytes":
+        if raw_value.startswith("0x"):
+            decoded_value = raw_value
+        else:
             decoded_value = "0x" + raw_value
 
-        elif argument_type == "byte":
-            decoded_value = "0x" + bytes.fromhex(raw_value[2:])[0].hex()
+    elif argument_type == "byte":
+        decoded_value = "0x" + bytes.fromhex(raw_value[2:])[0].hex()
 
-        elif argument_type in ("string", "string32"):
-            try:
-                if raw_value[:2] == "0x":
-                    raw_value = raw_value[2:]
-                decoded_value = (
-                    bytes.fromhex(raw_value).decode("utf-8").replace("\x00", "")
-                )
-            except Exception:
-                pass
+    elif argument_type in ("string", "string32"):
+        try:
+            if raw_value[:2] == "0x":
+                raw_value = raw_value[2:]
+            decoded_value = (
+                bytes.fromhex(raw_value).decode("utf-8").replace("\x00", "")
+            )
+        except Exception:
+            return raw_value
 
-        elif argument_type == "timestamp":
-            if isinstance(raw_value, str):
-                decoded_value = str(datetime.utcfromtimestamp(int(raw_value, 16)))
-            else:
-                decoded_value = str(datetime.utcfromtimestamp(raw_value))
+    elif argument_type == "timestamp":
+        if isinstance(raw_value, str):
+            decoded_value = str(datetime.utcfromtimestamp(int(raw_value, 16)))
+        else:
+            decoded_value = str(datetime.utcfromtimestamp(raw_value))
 
-        elif argument_type == "hashmap":
-            decoded_value = "[...]"
-        elif argument_type == "tuple":
-            decoded_value = "(...)"
-        elif argument_type == "tuple[]":
-            decoded_value = "(...)[]"
+    elif argument_type == "hashmap":
+        decoded_value = "[...]"
+    elif argument_type == "tuple":
+        decoded_value = "(...)"
+    elif argument_type == "tuple[]":
+        decoded_value = "(...)[]"
 
     return decoded_value
 
@@ -265,7 +271,7 @@ def decode_tuple(data, argument_abi, is_list):
         for c in range(count):
             do_offset = any(a.dynamic for a in argument_abi)
             if do_offset:
-                raw_value = data[c * 64 : (c + 1) * 64]
+                raw_value = data[c * 64: (c + 1) * 64]
                 offset = int(raw_value, 16) * 2
                 sub_bytes = data[offset:]
             else:
@@ -294,11 +300,13 @@ def decode_dynamic_array(data, array_type):
 
     for i in range(count):
         if array_type in ("bytes", "string"):
-            offset = int(sub_data[64 * i : 64 * (i + 1)], 16) * 2
+            offset = int(sub_data[64 * i: 64 * (i + 1)], 16) * 2
             decoded = decode_dynamic_argument(sub_data[offset:], array_type)
         else:
             offset = 64 * i
-            decoded = decode_static_argument(sub_data[offset : offset + 64], array_type)
+            if offset >= len(sub_data):
+                break
+            decoded = decode_static_argument(sub_data[offset: offset + 64], array_type)
 
         decoded_argument.append(decoded)
 
@@ -309,10 +317,11 @@ def decode_dynamic_array(data, array_type):
 def decode_dynamic_argument(argument_bytes, argument_type):
     if len(argument_bytes):
         length = int(argument_bytes[:64], 16) * 2
-        value = argument_bytes[64 : 64 + length]
+        value = argument_bytes[64: 64 + length]
 
         if argument_type == "string":
-            decoded_value = bytes.fromhex(value).decode("utf-8").replace("\x00", "")
+            hex_bytes = bytes.fromhex(value)
+            decoded_value = hex_bytes.decode("utf-8", "ignore").replace("\x00", "")
         else:
             decoded_value = "0x" + value
     else:
@@ -340,7 +349,7 @@ def decode_struct(data, arguments_abi):
                 else:
                     array_values.append(decode_static_argument(raw_value, array_type))
                     slot += 1
-                raw_value = data[slot * 64 : (slot + 1) * 64]
+                raw_value = data[slot * 64: (slot + 1) * 64]
 
         return array_values, slot
 
@@ -352,7 +361,7 @@ def decode_struct(data, arguments_abi):
     arguments_list = []
     slot = 0
     for i in range(no_arguments):
-        raw_value = data[slot * 64 : (slot + 1) * 64]
+        raw_value = data[slot * 64: (slot + 1) * 64]
 
         if arguments_abi:
 
@@ -367,7 +376,7 @@ def decode_struct(data, arguments_abi):
                     offset = int(raw_value, 16) * 2
                     sub_arguments = data[offset:]
                 else:
-                    sub_arguments = data[i * 64 :]
+                    sub_arguments = data[i * 64:]
 
                 argument_value, slots = decode_tuple(
                     sub_arguments,
