@@ -1,14 +1,18 @@
-#  Copyright 2021 DAI Foundation
+# Copyright 2021 DAI FOUNDATION (the original version https://github.com/daifoundation/ethtx_ce)
+# Copyright 2021-2022 Token Flow Insights SA (modifications to the original software as recorded
+# in the changelog https://github.com/EthTx/ethtx/blob/master/CHANGELOG.md)
 #
-#  Licensed under the Apache License, Version 2.0 (the "License");
-#  you may not use this file except in compliance with the License.
-#  You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
 #
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#  See the License for the specific language governing permissions and
-#  limitations under the License.
+# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
+# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and limitations under the License.
+#
+# The product contains trademarks and other branding elements of Token Flow Insights SA which are
+# not licensed under the Apache 2.0 license. When using or reproducing the code, please remove
+# the trademark and/or other branding elements.
 
 from __future__ import annotations
 
@@ -16,7 +20,7 @@ from datetime import datetime
 from typing import List, Any, Optional
 
 from ethtx.models.base_model import BaseModel
-from ethtx.models.objects_model import BlockMetadata, TransactionMetadata
+from ethtx.models.objects_model import BlockMetadata
 from ethtx.models.semantics_model import AddressSemantics, ERC20Semantics
 
 
@@ -33,6 +37,8 @@ class DecodedTransactionMetadata(BaseModel):
     block_hash: Optional[str]
     timestamp: Optional[datetime]
     gas_price: Optional[int]
+    from_address: Optional[str]
+    to_address: Optional[str]
     sender: Optional[AddressInfo]
     receiver: Optional[AddressInfo]
     tx_index: int
@@ -68,7 +74,7 @@ class DecodedCall(BaseModel):
     call_id: Optional[str]
     call_type: str
     from_address: AddressInfo
-    to_address: AddressInfo
+    to_address: Optional[AddressInfo]
     value: float
     function_signature: str
     function_name: str
@@ -98,7 +104,7 @@ class DecodedBalance(BaseModel):
 
 class DecodedTransaction(BaseModel):
     block_metadata: BlockMetadata
-    metadata: TransactionMetadata
+    metadata: DecodedTransactionMetadata
     events: List[DecodedEvent]
     calls: Optional[DecodedCall]
     transfers: List[DecodedTransfer]
