@@ -1,18 +1,25 @@
-#  Copyright 2021 DAI Foundation
+# Copyright 2021 DAI FOUNDATION (the original version https://github.com/daifoundation/ethtx_ce)
+# Copyright 2021-2022 Token Flow Insights SA (modifications to the original software as recorded
+# in the changelog https://github.com/EthTx/ethtx/blob/master/CHANGELOG.md)
 #
-#  Licensed under the Apache License, Version 2.0 (the "License");
-#  you may not use this file except in compliance with the License.
-#  You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
 #
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#  See the License for the specific language governing permissions and
-#  limitations under the License.
+# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
+# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and limitations under the License.
+#
+# The product contains trademarks and other branding elements of Token Flow Insights SA which are
+# not licensed under the Apache 2.0 license. When using or reproducing the code, please remove
+# the trademark and/or other branding elements.
+
+from __future__ import annotations
 
 from typing import List, Dict, Optional, TYPE_CHECKING
 
 from ethtx.models.base_model import BaseModel
+
 if TYPE_CHECKING:
     from ethtx.providers.semantic_providers import ISemanticsDatabase
 
@@ -26,11 +33,14 @@ class TransformationSemantics(BaseModel):
 class ParameterSemantics(BaseModel):
     parameter_name: str
     parameter_type: str
-    components: list = []
+    components: list[ParameterSemantics] = []
     indexed: bool = False
     dynamic: bool = False
 
 
+ParameterSemantics.update_forward_refs()
+
+
 class EventSemantics(BaseModel):
     signature: str
     anonymous: bool
@@ -85,9 +95,10 @@ class AddressSemantics(BaseModel):
     class Config:
         allow_mutation = True
 
-
     @staticmethod
-    def from_mongo_record(raw_address_semantics: Dict, database: 'ISemanticsDatabase') -> 'AddressSemantics':
+    def from_mongo_record(
+        raw_address_semantics: Dict, database: "ISemanticsDatabase"
+    ) -> "AddressSemantics":
 
         ZERO_HASH = "0xc5d2460186f7233c927e7db2dcc703c0e500b653ca82273b7bfad8045d85a470"
 
@@ -182,7 +193,6 @@ class AddressSemantics(BaseModel):
         chain_id = raw_address_semantics.get("chain_id")
         name = raw_address_semantics.get("name", address)
 
-
         address_semantics = AddressSemantics(
             chain_id=chain_id,
             address=address,
