@@ -1,18 +1,21 @@
-#  Copyright 2021 DAI Foundation
+# Copyright 2021 DAI FOUNDATION (the original version https://github.com/daifoundation/ethtx_ce)
+# Copyright 2021-2022 Token Flow Insights SA (modifications to the original software as recorded
+# in the changelog https://github.com/EthTx/ethtx/blob/master/CHANGELOG.md)
 #
-#  Licensed under the Apache License, Version 2.0 (the "License");
-#  you may not use this file except in compliance with the License.
-#  You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at: http://www.apache.org/licenses/LICENSE-2.0
 #
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#  See the License for the specific language governing permissions and
-#  limitations under the License.
+# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
+# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and limitations under the License.
+#
+# The product contains trademarks and other branding elements of Token Flow Insights SA which are
+# not licensed under the Apache 2.0 license. When using or reproducing the code, please remove
+# the trademark and/or other branding elements.
 
 import logging
 import os
-from functools import lru_cache
 from typing import List, Dict, Optional
 
 from web3 import Web3
@@ -23,8 +26,10 @@ from web3.types import BlockData, TxData, TxReceipt, HexStr
 from .node import NodeConnectionPool
 from ..exceptions import NodeConnectionException, ProcessingException
 from ..models.objects_model import Transaction, BlockMetadata, TransactionMetadata, Call
+from ..models.semantics_model import FunctionSemantics
 from ..models.w3_model import W3Block, W3Transaction, W3Receipt, W3CallTree, W3Log
 from ..semantics.standards import erc20
+from ..utils.cache_tools import cache
 
 log = logging.getLogger(__name__)
 
@@ -127,20 +132,25 @@ class Web3Provider(NodeDataProvider):
         ):
             w3 = connect_chain(http_hook=connection.url, poa=connection.poa)
 
-            if w3.isConnected():
-                log.info(
-                    "Connected to: %s with latest block %s.",
-                    connection,
-                    w3.eth.block_number,
+            try:
+                if w3.isConnected():
+                    log.info(
+                        "Connected to: %s with latest block %s.",
+                        connection,
+                        w3.eth.block_number,
+                    )
+                    return w3
+                else:
+                    log.warning("Connection failed to: %s", connection)
+            except AssertionError:
+                log.warning(
+                    "Node is available, but RPC connection failed: %s", connection
                 )
-                return w3
-            else:
-                log.warning("Connection failed to: %s", connection)
 
         raise NodeConnectionException
 
     # get the raw block data from the node
-    @lru_cache(maxsize=1024)
+    @cache
     def get_block(self, block_number: int, chain_id: Optional[str] = None) -> W3Block:
         chain = self._get_node_connection(chain_id)
         raw_block: BlockData = chain.eth.get_block(block_number)
@@ -170,7 +180,7 @@ class Web3Provider(NodeDataProvider):
         return block
 
     # get the raw transaction data from the node
-    @lru_cache(maxsize=1024)
+    @cache
     def get_transaction(
         self, tx_hash: str, chain_id: Optional[str] = None
     ) -> W3Transaction:
@@ -196,7 +206,7 @@ class Web3Provider(NodeDataProvider):
 
         return transaction
 
-    @lru_cache(maxsize=1024)
+    @cache
     def get_receipt(self, tx_hash: str, chain_id: Optional[str] = None) -> W3Receipt:
         chain = self._get_node_connection(chain_id)
         raw_receipt: TxReceipt = chain.eth.get_transaction_receipt(tx_hash)
@@ -239,11 +249,6 @@ class Web3Provider(NodeDataProvider):
 
         return receipt
 
-    @staticmethod
-    def _get_custom_calls_tracer():
-        return open(os.path.join(os.path.dirname(__file__), "static/tracer.js")).read()
-
-    @lru_cache(maxsize=1024)
     def get_calls(self, tx_hash: str, chain_id: Optional[str] = None) -> W3CallTree:
         # tracer is a temporary fixed implementation of geth tracer
         chain = self._get_node_connection(chain_id)
@@ -257,22 +262,22 @@ class Web3Provider(NodeDataProvider):
         )
 
     # get the contract bytecode hash from the node
-    @lru_cache(maxsize=1024)
+    @cache
     def get_code_hash(
         self, contract_address: str, chain_id: Optional[str] = None
     ) -> str:
         chain = self._get_node_connection(chain_id)
-        byte_code = chain.eth.get_code(Web3.toChecksumAddress(contract_address))
+        byte_code = chain.eth.get_code(Web3.toChecksumAddress(contract_address[-40:]))
         code_hash = Web3.keccak(byte_code).hex()
         return code_hash
 
     # get the erc20 token data from the node
-    @lru_cache(maxsize=1024)
+    @cache
     def get_erc20_token(
         self,
         token_address: str,
         contract_name: str,
-        functions,
+        functions: Dict[str, FunctionSemantics],
         chain_id: Optional[str] = None,
     ):
 
@@ -338,11 +343,13 @@ class Web3Provider(NodeDataProvider):
         return dict(address=token_address, symbol=symbol, name=name, decimals=decimals)
 
     # guess if the contract is and erc20 token and get the data
-    @lru_cache(maxsize=1024)
+    @cache
     def guess_erc20_token(self, contract_address, chain_id: Optional[str] = None):
         chain = self._get_node_connection(chain_id)
 
-        byte_code = chain.eth.get_code(Web3.toChecksumAddress(contract_address)).hex()
+        byte_code = chain.eth.get_code(
+            Web3.toChecksumAddress(contract_address[-40:])
+        ).hex()
 
         if all(
             "63" + signature[2:] in byte_code
@@ -395,7 +402,7 @@ class Web3Provider(NodeDataProvider):
         return None
 
     # guess if the contract is and erc20 token proxy and get the data
-    @lru_cache(maxsize=1024)
+    @cache
     def guess_erc20_proxy(self, contract_address, chain_id: Optional[str] = None):
         chain = self._get_node_connection(chain_id)
 
@@ -430,7 +437,7 @@ class Web3Provider(NodeDataProvider):
         return None
 
     # guess if the contract is and erc721 token proxy and get the data
-    @lru_cache(maxsize=1024)
+    @cache
     def guess_erc721_proxy(self, contract_address, chain_id: Optional[str] = None):
         chain = self._get_node_connection(chain_id)
 
@@ -459,7 +466,7 @@ class Web3Provider(NodeDataProvider):
 
         return None
 
-    @lru_cache(maxsize=1024)
+    @cache
     def get_full_transaction(self, tx_hash: str, chain_id: Optional[str] = None):
 
         w3transaction = self.get_transaction(tx_hash, chain_id)
@@ -470,9 +477,11 @@ class Web3Provider(NodeDataProvider):
             w3transaction=w3transaction, w3receipt=w3receipt, w3calltree=w3calltree
         )
 
-    @staticmethod
+    def _get_custom_calls_tracer(self):
+        return open(os.path.join(os.path.dirname(__file__), "static/tracer.js")).read()
+
     def _create_call_from_debug_trace_tx(
-        tx_hash: str, chain_id: str, input_rpc: AttributeDict
+        self, tx_hash: str, chain_id: str, input_rpc: AttributeDict
     ) -> W3CallTree:
         def prep_raw_dict(dct: [AttributeDict, Dict]):
             if not isinstance(dct, dict):
